// 2011.03.29.L -> Implemented Refresh Limitting
// 2011.03.26.L -> Modified the Laser Tracker to keep track of two laser dots
// 2011.02.24.L -> Implememnted the preliminary version of the laser tracker

/*###############################################
##      
##      Laser Tracking for XUPV2P (laser_track)
##
##      Verilog Source File (.v)
##
##      Based on video_to_ram system created by Jeffrey Goeders
##	     Laser tracking created by Latch Dimitrov
##
##      
##
###############################################*/

module video_to_ram
(
	// System
	i_sys_rst,
	i_clk_100_bus,

	// Video Input
	i_LLC_CLOCK,    
	i_YCrCb,

	// Bus Master
	o_M_abort,
	o_M_ABus,
	o_M_BE,
	o_M_busLock,
	o_M_TAttribute,
	o_M_lockErr,
	o_M_mSize,
	o_M_priority,
	o_M_rdBurst,
	o_M_request,
	o_M_RNW,
	o_M_size,
	o_M_type,
	o_M_wrBurst,
	o_M_wrDBus,
	i_PLB_MAddrAck,
	i_PLB_MBusy,
	i_PLB_MRdErr,
	i_PLB_MWrErr,
	i_PLB_MRdBTerm,
	i_PLB_MRdDAck,
	i_PLB_MRdDBus,
	i_PLB_MRdWdAddr,
	i_PLB_MRearbitrate,
	i_PLB_MSSize,
	i_PLB_MWrBTerm,
	i_PLB_MWrDAck,
	i_PLB_MTimeout,
	i_PLB_PAValid,
	
	i_Sl_addrAck,
	i_Sl_wrDAck,
	i_Sl_wrComp,
	
	i_Sl_rdDBus,
	i_Sl_rdDAck,
	i_Sl_rdComp,

	//MPMC
	i_MPMC_Done_Init,

	//Debug
	o_DBG_fsm_cs,
	o_DBG_general_purpose,
	i_DBG_new_line
);
	
	////////////////////////////////////////////////////////////
	////////////////////////// PARAMETERS //////////////////////
	////////////////////////////////////////////////////////////
	
	//Defines
	`define rcc						r_current_color_limits

	`define limR					{ `rcc[31], `rcc[30], `rcc[29], `rcc[28], `rcc[27], `rcc[26] }
	`define limG					{ `rcc[25], `rcc[24], `rcc[23], `rcc[22], `rcc[21], `rcc[20] }
	`define limB					{ `rcc[19], `rcc[18], `rcc[17], `rcc[16], `rcc[15], `rcc[14] }
	`define marg					{ `rcc[13], `rcc[12], `rcc[11], `rcc[10], `rcc[9], `rcc[8] }
	`define refresh_rate 		{ `rcc[7], `rcc[6], `rcc[5], `rcc[4], `rcc[3], `rcc[2], `rcc[1], `rcc[0] }
	
	// Assigned by Xilinx Tools
	parameter   					C_PLBV46_NUM_SLAVES 	= 1;
	parameter   					C_PLBV46_DWIDTH     	= 64;
	
	// Inherited From MPD file
	parameter   					C_VIDEO_RAM_BASEADDR	= 32'h00000000;
	parameter						C_BYTES_PER_LINE		= 4096;

	// Local
	localparam						C_PIXELS_PER_LINE		= 11'd640;
	localparam						C_NUM_LINES				= 10'd480;
	
	localparam   					C_COLOR_LIMITS_ADDR	= 32'h45000000;	// The address location of the colour limits word
	localparam						C_CURRENT_POS_1_ADDR	= 32'h45000004;	// The address location of the current laser 1 position word
	localparam						C_CURRENT_POS_2_ADDR	= 32'h45000008;	// The address location of the current laser 2 position word

    //FSM States
	localparam	C_STATE_BITS = 4;
	localparam	[C_STATE_BITS-1:0]	S_INIT 			      = 0,
												S_START_LINE			= 1,
												
												S_LINE_HOLDER			= 2,
												
												S_READ_REQUEST			= 3, 		//Reading the track colour limits
												S_READ					= 4,		//Reading the track colour limits
												S_READ_COMPLETE		= 5,		//Reading the track colour limits
												
												S_WRITE_REQUEST_1		= 6,		//Writing the current laser 1 coordinates
												S_WRITE_1				= 7,		//Writing the current laser 1 coordinates
												S_WRITE_COMPLETE_1	= 8,		//Writing the current laser 1 coordinates
												
												S_WRITE_REQUEST_2		= 9,		//Writing the current laser 2 coordinates
												S_WRITE_2				= 10,		//Writing the current laser 2 coordinates
												S_WRITE_COMPLETE_2	= 11;		//Writing the current laser 2 coordinates
												
	/*
	In Simulation Debug mode, external inputs are used for some of the
	video timing signals as actually simulating video input is difficult
	and time consuming.

	Specifically, the H_444 (new line) signal used to indicate that a switch
	between the buffers has occurred becomes externally driven for debugging
	purposes.  Also the data written to RAM (c_M_wrDBus) is driven to 
	a constant debug value of 0x12345678.
	*/
	localparam                      C_SIMULATION_DEBUG_MODE = 0;

	////////////////////////////////////////////////////////////
	////////////////////////// PORTS ///////////////////////////
	////////////////////////////////////////////////////////////

	// System
	input                           		i_sys_rst;
	input                           		i_clk_100_bus;

	// Video 
	input                           		i_LLC_CLOCK;     	// Line Locked Clock (27MHz) from VDEC1 daughter board
	input   [9:2]                   		i_YCrCb;            // YCrCb digital video data from VDEC1 daughter board

	// PLB Master
	output                                  o_M_abort;
	output  [0:31]                          o_M_ABus;
	output  [0:3]				                o_M_BE;
	output                                  o_M_busLock;
	output  [0:15]                          o_M_TAttribute;
	output                                  o_M_lockErr;
	output  [0:1]                           o_M_mSize;
	output  [0:1]                           o_M_priority;
	output                                  o_M_rdBurst;
	output                                  o_M_request;
	output                                  o_M_RNW;
	output  [0:3]                           o_M_size;
	output  [0:2]                           o_M_type;
	output                                  o_M_wrBurst;
	output  [0:(C_PLBV46_DWIDTH-1)]         o_M_wrDBus;
	input                                   i_PLB_MAddrAck;
	input                                   i_PLB_MBusy;
	input                                   i_PLB_MRdErr;
	input                                   i_PLB_MWrErr;
	input                                   i_PLB_MRdBTerm;
	input                                   i_PLB_MRdDAck;
	input   [0:(C_PLBV46_DWIDTH-1)]         i_PLB_MRdDBus;
	input   [0:3]                           i_PLB_MRdWdAddr;
	input                                   i_PLB_MRearbitrate;
	input   [0:1]                           i_PLB_MSSize;
	input                                   i_PLB_MWrBTerm;
	input                                   i_PLB_MWrDAck;
	input                                   i_PLB_MTimeout;
	input                                   i_PLB_PAValid;
	
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_addrAck;
	
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_wrDAck;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_wrComp;
	
	input   [0:(C_PLBV46_DWIDTH-1)]         i_Sl_rdDBus;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_rdDAck;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_rdComp;

	//MPMC
	input                                   i_MPMC_Done_Init;

	//Debug
	input                                   i_DBG_new_line;
	output  [C_STATE_BITS-1:0]              o_DBG_fsm_cs;
	output  [3:0]                           o_DBG_general_purpose;
    
	////////////////////////////////////////////////////////////
	////////////////////////// DECLARATIONS ////////////////////
	////////////////////////////////////////////////////////////

    //Clocks
	wire									  clk_13;
	wire                            clk_27;
	wire                            clk_ceo_444;

    //Input Video Processing
	wire    [9:0]                   w_concat_YCrCb;
	wire    [9:0]                   w_YCrCb_422;
	wire                            w_NTSC_out;
	wire                            w_F_422;
	wire                            w_V_422;
	wire                            w_H_422;
	wire                            w_F_444;
	wire                            w_V_444;
	wire                            w_H_444;
   reg                             w_H_444_with_DBG;
	wire    [9:0]                   w_Y;
	wire    [9:0]                   w_Cr;
	wire    [9:0]                   w_Cb;

	//Line Buffers
	reg                             r_write_buffer_0;
	wire                            w_read_buffer_0;
	wire                            w_write_buffer_0;
	wire                            w_read_buffer_1;
	wire                            w_write_buffer_1;
	reg     [10:0]                  r_line_buffer_write_addr; 
	wire    [7:0]                   w_line_buffer_read_Red_0;
	wire    [7:0]                   w_line_buffer_read_Green_0;
	wire    [7:0]                   w_line_buffer_read_Blue_0;
	wire    [7:0]                   w_line_buffer_read_Red_1;
	wire    [7:0]                   w_line_buffer_read_Green_1;
	wire    [7:0]                   w_line_buffer_read_Blue_1;
	wire    [7:0]                   w_Red_to_buffer;
	wire    [7:0]                   w_Green_to_buffer;
	wire    [7:0]                   w_Blue_to_buffer;

   reg                             r_line_ready;
    
	//Special Timing Generation
	wire							w_one_shot_out;
	wire							w_rst_timing_generation;
	wire	[9:0]					w_vga_timing_line_count;

	// Memory Access State Machine
	reg		[C_STATE_BITS-1:0]		r_cs;
	reg		[C_STATE_BITS-1:0]		c_ns;
	
	//PLB Master General
	wire										w_M_request;
	reg		[0:31]						r_M_ABus;
	wire										w_M_RNW;

   // PLB Master Write
	
	//Laser Tracking
	reg 		[10:0]	r_pixel_count;				   // The line pixel counter
	reg      [0:31]	r_current_color_limits;		// The current color limits word
	reg		[7:0]		r_refresh_counter;			//The refresh counter
	reg					trap;
	reg		[9:0]		r_frame_counter;				// The frame counter			
	reg      [0:31]	r_current_laser_1_pos;		// The coordinates of laser 1 in the current frame word
	reg      [0:31]	r_current_laser_2_pos;		// The coordinates of laser 1 in the current frame word
	reg					trap_first_laser;
	
	//Debug
	reg     [C_STATE_BITS-1:0]      c_DBG_fsm_cs;
	wire    [3:0]                   w_DBG_general_purpose;

	////////////////////////////////////////////////////////////
	////////////////////////// ASSIGNMENTS /////////////////////
	////////////////////////////////////////////////////////////

    // Video Processing
	assign 	w_concat_YCrCb 			= {i_YCrCb[9:2],2'B00};	
	
    // Line Buffers
	assign 	w_read_buffer_0			= !r_write_buffer_0;
	assign 	w_write_buffer_0			= r_write_buffer_0;
	assign	w_read_buffer_1			= r_write_buffer_0;
	assign 	w_write_buffer_1			= !r_write_buffer_0;

	// Special Timing Generation
	assign 	w_rst_timing_generation	= ( i_sys_rst || w_one_shot_out );
    
   // Output to PLB Bus
	assign 	o_M_abort				= 1'b0;						// Not Used
	assign	o_M_ABus					= r_M_ABus;
	assign	o_M_BE					= 4'hF;						// Byte Enable is 4'b1111
	assign	o_M_busLock				= 1'b0;						// Not Used
	assign	o_M_TAttribute			= 16'b0;						// Not Used
	assign 	o_M_lockErr				= 1'b0;						// Not Used
	assign 	o_M_mSize				= 2'b0;						// 32-bit bus size
	assign 	o_M_priority			= 2'b11;						// Always highest priority
	assign 	o_M_rdBurst				= 1'b0;						// Never requesting burst read operations
	assign 	o_M_request				= w_M_request;
	assign 	o_M_RNW					= w_M_RNW;
	assign 	o_M_size					= 4'h0;						// No Burst
	assign 	o_M_type					= 3'b000;					// Standard Memory Transfer
	assign	o_M_wrBurst				= 1'b0;						// Never requesting burst write operations
	assign 	o_M_wrDBus[0:31]		= ( (r_cs == S_WRITE_REQUEST_2) ||
													(r_cs == S_WRITE_2) ||
													(r_cs == S_WRITE_COMPLETE_2) ) ? r_current_laser_2_pos[0:31] : r_current_laser_1_pos[0:31];	//Writing the current laser position (1 or 2)
	
	// PLB Signal Logic
	assign	w_M_request				= ( (r_cs == S_READ_REQUEST) || (r_cs == S_WRITE_REQUEST_1) || (r_cs == S_WRITE_REQUEST_2) );
	assign	w_M_RNW					= (r_cs == S_READ_REQUEST);

	//Debug 
	assign  w_DBG_general_purpose   = { r_current_color_limits[31], r_current_color_limits[30], r_current_color_limits[29], r_current_color_limits[28] };
	assign  o_DBG_fsm_cs            = c_DBG_fsm_cs;
	assign  o_DBG_general_purpose   = w_DBG_general_purpose;

	////////////////////////////////////////////////////////////
	///////////////////// COMBINATIONAL LOGIC //////////////////
	////////////////////////////////////////////////////////////

	// c_ns
	always @ (*)
	begin
		case(r_cs)
		S_INIT:
		begin
			if ( i_MPMC_Done_Init == 1'b1 )
				c_ns <= S_START_LINE;
			else
				c_ns <= S_INIT;
		end
		S_START_LINE:
		begin
            if ( r_line_ready == 1'b1)
                c_ns <= S_LINE_HOLDER;
            else
                c_ns <= S_START_LINE;
		end
		S_LINE_HOLDER:
		begin
			if ( r_pixel_count < C_PIXELS_PER_LINE - 1 )
				c_ns <= S_LINE_HOLDER;
			else if ( w_vga_timing_line_count == C_NUM_LINES - 1 )
			begin
				if ( r_refresh_counter == `refresh_rate )
					c_ns <= S_READ_REQUEST;
				else
					c_ns <= S_START_LINE;
			end
			else
				c_ns <= S_START_LINE;
		end
		S_READ_REQUEST:
		begin
			if ( i_Sl_addrAck == 1'b1 && i_PLB_PAValid == 1'b1 )
				c_ns <= S_READ;
			else
				c_ns <= S_READ_REQUEST;
		end
		S_READ:
		begin
			if ( i_Sl_rdComp == 1'b1 )
				c_ns <= S_READ_COMPLETE;
			else
				c_ns <= S_READ;
		end
		S_READ_COMPLETE:
		begin			
				c_ns <= S_WRITE_REQUEST_1;
		end
		S_WRITE_REQUEST_1:
		begin
			if ( i_Sl_addrAck == 1'b1 && i_PLB_PAValid == 1'b1 )
				c_ns <= S_WRITE_1;
			else
				c_ns <= S_WRITE_REQUEST_1;
		end
		S_WRITE_1:
		begin
			if ( i_Sl_wrComp == 1'b1 )
				c_ns <= S_WRITE_COMPLETE_1;
			else
				c_ns <= S_WRITE_1;
		end
		S_WRITE_COMPLETE_1:
		begin			
				c_ns <= S_WRITE_REQUEST_2;
		end
		S_WRITE_REQUEST_2:
		begin
			if ( i_Sl_addrAck == 1'b1 && i_PLB_PAValid == 1'b1 )
				c_ns <= S_WRITE_2;
			else
				c_ns <= S_WRITE_REQUEST_2;
		end
		S_WRITE_2:
		begin
			if ( i_Sl_wrComp == 1'b1 )
				c_ns <= S_WRITE_COMPLETE_2;
			else
				c_ns <= S_WRITE_2;
		end
		S_WRITE_COMPLETE_2:
		begin			
				c_ns <= S_START_LINE;
		end
		default:
		begin
			c_ns <= S_INIT;
		end
		endcase
	end

    //c_DBG_fsm_cs
    always @ (*)
    begin
        case (r_cs)
        S_INIT:
        begin
            c_DBG_fsm_cs <= 0;
        end        
        S_START_LINE:
        begin
            c_DBG_fsm_cs <= 1;  
        end
        S_LINE_HOLDER:
        begin
            c_DBG_fsm_cs <= 2;
        end
        S_READ_REQUEST:
        begin
            c_DBG_fsm_cs <= 3;
        end
        S_READ:
        begin
            c_DBG_fsm_cs <= 4;
        end
        S_READ_COMPLETE:
        begin
            c_DBG_fsm_cs <= 5;
        end
        S_WRITE_REQUEST_1:
        begin
            c_DBG_fsm_cs <= 6;
        end
        S_WRITE_1:
        begin
            c_DBG_fsm_cs <= 7;
        end
        S_WRITE_COMPLETE_1:
        begin
            c_DBG_fsm_cs <= 8;
        end
        S_WRITE_REQUEST_2:
        begin
            c_DBG_fsm_cs <= 9;
        end
        S_WRITE_2:
        begin
            c_DBG_fsm_cs <= 10;
        end
        S_WRITE_COMPLETE_2:
        begin
            c_DBG_fsm_cs <= 11;
        end
        default
        begin
            c_DBG_fsm_cs <= 12;
        end
        endcase
    end
    
   //w_H_444_with_DBG
	always @ (*)
	begin
	  if ( C_SIMULATION_DEBUG_MODE == 1 )
			w_H_444_with_DBG    <= i_DBG_new_line;
	  else
			w_H_444_with_DBG    <= w_H_444;
	end
    
	////////////////////////////////////////////////////////////
	///////////////////// CLOCKED ASSIGNMENTS //////////////////
	////////////////////////////////////////////////////////////

	// r_cs
	always @ (posedge i_sys_rst or posedge i_clk_100_bus)
	begin
		if ( i_sys_rst == 1'b1 )
		begin
			r_cs <= S_INIT;
		end
		else
		begin
			r_cs <= c_ns;
		end
	end

	// r_write_buffer_0
	always @ (posedge w_H_444_with_DBG or posedge i_sys_rst) 
	begin
		if ( i_sys_rst == 1'b1 ) 
		begin
			r_write_buffer_0 <= 1'b1;
		end
		else
		begin
			r_write_buffer_0 <= !r_write_buffer_0;
		end
	end
	   
	// r_line_buffer_write_addr
	always @ (posedge clk_13 or posedge w_H_444_with_DBG) 
	begin
		if ( w_H_444_with_DBG == 1'b1 ) 
		begin
			r_line_buffer_write_addr <= 11'h000;
		end
		else 
		begin
			r_line_buffer_write_addr <= r_line_buffer_write_addr + 1;
		end
	end

	//r_pixel_count
	always @ (posedge i_clk_100_bus)
	begin
		if ( r_cs == S_START_LINE )
			r_pixel_count <= 11'h0;
		else if ( (r_cs == S_LINE_HOLDER) && (c_ns == S_LINE_HOLDER) )
			r_pixel_count <= r_pixel_count + 1;
		else 
			r_pixel_count <= r_pixel_count;
	end
    
  //r_line_ready
  always @ (posedge w_H_444_with_DBG or posedge i_clk_100_bus)
  begin
		if ( w_H_444_with_DBG == 1'b1)
			r_line_ready <= 1'b1;
		else if ( c_ns == S_START_LINE )
			r_line_ready <= 1'b0;
		else
			r_line_ready <= r_line_ready;
  end
  
	//r_M_ABus
	always @ (posedge i_clk_100_bus)
	begin
		if (c_ns == S_READ_REQUEST)
			r_M_ABus[0:31] <= C_COLOR_LIMITS_ADDR;
		else if (c_ns == S_WRITE_REQUEST_1)
			r_M_ABus[0:31] <= C_CURRENT_POS_1_ADDR;
		else if (c_ns == S_WRITE_REQUEST_2)
			r_M_ABus[0:31] <= C_CURRENT_POS_2_ADDR;
		else
			r_M_ABus <= r_M_ABus;
	end
	
	//r_current_color_limits
	always @ (posedge i_clk_100_bus)
	begin
		if ( r_cs == S_INIT )
			r_current_color_limits[0:31] <= 32'h0;
		else if (i_Sl_rdDAck)
			r_current_color_limits[0:31] <= i_Sl_rdDBus[0:31];
		else
			r_current_color_limits <= r_current_color_limits;
	end
	
	//r_refresh_counter
	always @ (posedge i_clk_100_bus)
	begin
		if ( (r_cs == S_INIT) || (r_cs == S_WRITE_COMPLETE_2) )
			r_refresh_counter <= 8'h0;
		else if ( (r_cs == S_LINE_HOLDER) && (r_pixel_count == C_PIXELS_PER_LINE - 1) && (w_vga_timing_line_count == C_NUM_LINES - 1) )
			r_refresh_counter <= r_refresh_counter + 1;
		else
			r_refresh_counter <= r_refresh_counter;
	end
	
	//trap
	always @ (posedge i_clk_100_bus)
	begin
		if ( w_vga_timing_line_count == 10'h0 )
			trap <= 1'b1;
		else
			trap <= 1'b0;
	end
	
	//r_frame_counter
	always @ (posedge i_clk_100_bus)
	begin
		if ( (w_vga_timing_line_count == 10'h0) && (trap == 1'b0) )
			r_frame_counter <= r_frame_counter + 1;
		else
			r_frame_counter <= r_frame_counter;
	end
	
	//r_current_laser_1_pos, r_current_laser_2_pos and trap_first_laser
	always @ (posedge i_clk_100_bus)
	begin
		if ( (w_vga_timing_line_count == 10'h0) && (trap == 1'b0) )
		begin
			r_current_laser_1_pos[0:31] 	<= { (r_frame_counter + 1), 22'h0 };
			r_current_laser_2_pos[0:31] 	<= { (r_frame_counter + 1), 22'h0 };
			trap_first_laser <= 1'b0;
		end
		else
		begin
			if (r_cs == S_LINE_HOLDER)
			begin
				if ( w_read_buffer_0 == 1'b1 )
				begin
						if	( ( {2'b00, w_line_buffer_read_Red_0[7:2]} < {2'b00, `limR} + {2'b00, `marg} ) && ( {2'b00, w_line_buffer_read_Red_0[7:2]} > {2'b00, `limR} - {2'b00, `marg} ) &&
							  ( {2'b00, w_line_buffer_read_Green_0[7:2]} < {2'b00, `limG} + {2'b00, `marg} ) && ( {2'b00, w_line_buffer_read_Green_0[7:2]} > {2'b00, `limG} - {2'b00, `marg} ) &&
							  ( {2'b00, w_line_buffer_read_Blue_0[7:2]} < {2'b00, `limB} + {2'b00, `marg} ) && ( {2'b00, w_line_buffer_read_Blue_0[7:2]} > {2'b00, `limB} - {2'b00, `marg}) )
						begin
							if ( trap_first_laser == 1'b0 )
							begin
								r_current_laser_1_pos[0:31] <= { r_frame_counter, w_vga_timing_line_count, 1'b0, r_pixel_count };
								r_current_laser_2_pos[0:31] <= r_current_laser_2_pos[0:31];
								trap_first_laser <= 1'b1;
							end
							else
							begin
								r_current_laser_1_pos[0:31] <= r_current_laser_1_pos[0:31];
								r_current_laser_2_pos[0:31] <= { r_frame_counter, w_vga_timing_line_count, 1'b0, r_pixel_count };
								trap_first_laser <= trap_first_laser;
							end
						end
						else
						begin
							r_current_laser_1_pos[0:31] <= r_current_laser_1_pos[0:31];
							r_current_laser_2_pos[0:31] <= r_current_laser_2_pos[0:31];
							trap_first_laser <= trap_first_laser;
						end
				end
				else
				begin
						if	( ( {2'b00, w_line_buffer_read_Red_1[7:2]} < {2'b00, `limR} + {2'b00, `marg} ) && ( {2'b00, w_line_buffer_read_Red_1[7:2]} > {2'b00, `limR} - {2'b00, `marg} ) &&
							  ( {2'b00, w_line_buffer_read_Green_1[7:2]} < {2'b00, `limG} + {2'b00, `marg} ) && ( {2'b00, w_line_buffer_read_Green_1[7:2]} > {2'b00, `limG} - {2'b00, `marg} ) &&
							  ( {2'b00, w_line_buffer_read_Blue_1[7:2]} < {2'b00, `limB} + {2'b00, `marg} ) && ( {2'b00, w_line_buffer_read_Blue_1[7:2]} > {2'b00, `limB} - {2'b00, `marg}) )
						begin
							if ( trap_first_laser == 1'b0 )
							begin
								r_current_laser_1_pos[0:31] <= { r_frame_counter, w_vga_timing_line_count, 1'b0, r_pixel_count };
								r_current_laser_2_pos[0:31] <= r_current_laser_2_pos[0:31];
								trap_first_laser <= 1'b1;
							end
							else
							begin
								r_current_laser_1_pos[0:31] <= r_current_laser_1_pos[0:31];
								r_current_laser_2_pos[0:31] <= { r_frame_counter, w_vga_timing_line_count, 1'b0, r_pixel_count };
								trap_first_laser <= trap_first_laser;
							end
						end
						else
						begin
							r_current_laser_1_pos[0:31] <= r_current_laser_1_pos[0:31];
							r_current_laser_2_pos[0:31] <= r_current_laser_2_pos[0:31];
							trap_first_laser <= trap_first_laser;
						end
				end
			end
			else
			begin
				r_current_laser_1_pos[0:31] <= r_current_laser_1_pos[0:31];
				r_current_laser_2_pos[0:31] <= r_current_laser_2_pos[0:31];
				trap_first_laser <= trap_first_laser;
			end
		end
	end
	
	////////////////////////////////////////////////////////////
	////////////////////////// INSTANTIATIONS //////////////////
	////////////////////////////////////////////////////////////

	// instantiate the clock input buffers for the line locked clock
	IBUFG LLC_INPUT_BUF 
	(
		.O (clk_27),
		.I (i_LLC_CLOCK)
	);

	// instantiate the clock input buffers for the internal clocks
	BUFG CLK_13MHZ_BUF 
	(
		.O  (clk_13),
		.I  (clk_ceo_444)
	);

	// instantiare a DDR output flip flop to center the clock in the pixel data
	OFDDRRSE  PIXEL_CLOCK_DDR_FF
	(
		.Q   (),
		.C0  (clk_27),
		.C1  (!clk_27),
		.CE  (1'b1),
		.D0  (1'b0),
		.D1  (1'b1),
		.R   (),
		.S   (1'b0)
	);

	// INSTANTIATE THE LINE FIELD DECODER
	// THIS EXTRACTS THE H,V,F BITS FROM THE
	// TIMING REFERENCE CODES
	lf_decode lf_decode_i
	(
		.rst       (i_sys_rst),             // Reset and Clock input
		.clk       (clk_27),                // 27Mhz for SDTV
		.YCrCb_in  (w_concat_YCrCb),        // data from the input video stream
		.YCrCb_out (w_YCrCb_422),           // data delayed by pipe length
		.NTSC_out  (w_NTSC_out),            // high = NTSC format detected
		.Fo        (w_F_422),               // high = field one (even)
		.Vo        (w_V_422),               // high = vertical blank
		.Ho        (w_H_422)                // low = active video
	);

	// CONVERT FROM 4:2:2 DATA TO 4:4:4 DATA
	vp422_444_dup vp422_444_dup_i
	(
		.rst          (i_sys_rst),          // resets input data register and control
		.clk          (clk_27),             // video component rate clock, 27Mhz for SDTV
		.ycrcb_in     (w_YCrCb_422),        // video component data
		.ntsc_in      (w_NTSC_out),     
		.fi           (w_F_422),            // Low to High signals start of Field One
		.vi           (w_V_422),            // High signals Vertical Blanking
		.hi           (w_H_422),            // High signals Horizontal Blanking
		.ceo          (clk_ceo_444),        // output data rate is 1/2 the clock rate
		.ntsc_out_o   (),   
		.fo           (w_F_444),            // Field signal delayed by pipe length
		.vo           (w_V_444),            // Vertical signal delayed by pipe length
		.ho           (w_H_444),            // Horizontal signal delayed by pipe length
		.y_out        (w_Y),                // Y out
		.cr_out       (w_Cr),               // Cr out
		.cb_out       (w_Cb)                // Cb out
	);

	// INSTANTIATE THE COLOR SPACE CONVERTER
	YCrCb2RGB YCrCb2RGB_i
	(
		.R    (w_Red_to_buffer),       
		.G    (w_Green_to_buffer),     
		.B    (w_Blue_to_buffer),      
		.clk  (clk_13),    
		.rst  (i_sys_rst),       
		.Y    (w_Y),  
		.Cr   (w_Cr), 
		.Cb   (w_Cb)  
	); 

	// INSTANTIATE TWO VIDEO LINE BUFFERS. THESE LINE BUFFERS SWITCH
	// FROM READ TO WRITE AT THE END OF EACH HORIZONTAL BLANKING INTERVAL
	// THESE BUFFERS RESIDE IN BRAMs
	LINE_BUFFER LB0
	(
	.read_clk        (i_clk_100_bus),
	.read_address    (r_pixel_count + 11'h00A),
	.read_enable     (w_read_buffer_0),
	.read_red_data   (w_line_buffer_read_Red_0),
	.read_green_data (w_line_buffer_read_Green_0),
	.read_blue_data  (w_line_buffer_read_Blue_0),

	.write_clk        (clk_13),
	.write_address    (r_line_buffer_write_addr), 
	.write_enable     (w_write_buffer_0),
	.write_red_data   (w_Red_to_buffer),
	.write_green_data (w_Green_to_buffer),
	.write_blue_data  (w_Blue_to_buffer)
	);

	LINE_BUFFER LB1
	(
	.read_clk         (i_clk_100_bus),
	.read_address     (r_pixel_count + 11'h00A),
	.read_enable      (w_read_buffer_1),
	.read_red_data    (w_line_buffer_read_Red_1),
	.read_green_data  (w_line_buffer_read_Green_1),
	.read_blue_data   (w_line_buffer_read_Blue_1),

	.write_clk        (clk_13),
	.write_address    (r_line_buffer_write_addr),  
	.write_enable     (w_write_buffer_1),
	.write_red_data   (w_Red_to_buffer),
	.write_green_data (w_Green_to_buffer),
	.write_blue_data  (w_Blue_to_buffer)
	);

	// INSTANTIATE A ONE SHOT TO DETECT THE FALLING EDGS OF THE FIELD
	// BIT IN THE TIMING REFERENCE CODE. THIS PULSE RESETS THE
	// SVGA TIMING GENERATION MODULE TO SYNCH THE OUTPUT VIDEO WITH
	// THE VIDEO SOURCE
	NEG_EDGE_DETECT NEG_EDGE_DETECT_i
	(
		.clk          (clk_27),     
		.data_in      (w_F_444),     
		.reset        (i_sys_rst),        
		.one_shot_out (w_one_shot_out)
	);

	// INSTANTIATE THE SVGA TIMING GENERATION MODULE
	// This module is used for the line count generates
	// It handles the interlaced video and correctly outputs the 
	// appropriate line numbere
	SPECIAL_SVGA_TIMING_GENERATION SPECIAL_SVGA_TIMING_GENERATION
	(
		.pixel_clock          (clk_27),           
		.reset                (w_rst_timing_generation), 
		.h_synch_delay        (),    
		.v_synch_delay        (),    
		.comp_synch           (),       
		.blank                (),            
		.char_line_count      (),  
		.char_address         (),     
		.char_pixel           (),       
		.pixel_count          (),      
		.line_count_out       (w_vga_timing_line_count)
	);
	
endmodule

module IBUFG(O, I); // synthesis syn_black_box
	output O;
	input I;
endmodule

module BUFG(O, I); // synthesis syn_black_box
	output O;
	input I;
endmodule

module OFDDRRSE (Q, C0, C1, CE, D0, D1, R, S); // synthesis syn_black_box
    output Q;
    input  C0, C1, CE, D0, D1, R, S;
endmodule
